@@ -5,9 +5,9 @@
 
 #include "mpiimpl.h"
 
-int MPIR_Gather_allcomm_nb(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf,
-                           int recvcount, MPI_Datatype recvtype, int root, MPIR_Comm * comm_ptr,
-                           MPIR_Errflag_t * errflag)
+int MPIR_Gather_allcomm_nb(const void *sendbuf, MPI_Aint sendcount, MPI_Datatype sendtype,
+                           void *recvbuf, MPI_Aint recvcount, MPI_Datatype recvtype, int root,
+                           MPIR_Comm * comm_ptr, MPIR_Errflag_t * errflag)
 {
     int mpi_errno = MPI_SUCCESS;
     MPIR_Request *req_ptr = NULL;
