@@ -1,11 +1,11 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
-/* 
- *
- *   Copyright (C) 1997 University of Chicago. 
- *   See COPYRIGHT notice in top-level directory.
+/*
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
 
 #include "mpioimpl.h"
+#include <limits.h>
+#include <assert.h>
 
 #ifdef HAVE_WEAK_SYMBOLS
 
@@ -18,7 +18,19 @@
 /* end of weak pragmas */
 #elif defined(HAVE_WEAK_ATTRIBUTE)
 int MPI_File_read_all_begin(MPI_File fh, void *buf, int count, MPI_Datatype datatype)
-    __attribute__((weak,alias("PMPI_File_read_all_begin")));
+    __attribute__ ((weak, alias("PMPI_File_read_all_begin")));
+#endif
+
+#if defined(HAVE_PRAGMA_WEAK)
+#pragma weak MPI_File_read_all_begin_c = PMPI_File_read_all_begin_c
+#elif defined(HAVE_PRAGMA_HP_SEC_DEF)
+#pragma _HP_SECONDARY_DEF PMPI_File_read_all_begin_c MPI_File_read_all_begin_c
+#elif defined(HAVE_PRAGMA_CRI_DUP)
+#pragma _CRI duplicate MPI_File_read_all_begin_c as PMPI_File_read_all_begin_c
+/* end of weak pragmas */
+#elif defined(HAVE_WEAK_ATTRIBUTE)
+int MPI_File_read_all_begin_c(MPI_File fh, void *buf, MPI_Count count, MPI_Datatype datatype)
+    __attribute__ ((weak, alias("PMPI_File_read_all_begin_c")));
 #endif
 
 /* Include mapping from MPI->PMPI */
@@ -39,15 +51,41 @@ Output Parameters:
 
 .N fortran
 @*/
-int MPI_File_read_all_begin(MPI_File fh, void *buf, int count,
-                            MPI_Datatype datatype)
+int MPI_File_read_all_begin(MPI_File fh, void *buf, int count, MPI_Datatype datatype)
 {
     int error_code;
     static char myname[] = "MPI_FILE_READ_ALL_BEGIN";
 
     error_code = MPIOI_File_read_all_begin(fh, (MPI_Offset) 0,
-					   ADIO_INDIVIDUAL, buf, count,
-					   datatype, myname);
+                                           ADIO_INDIVIDUAL, buf, count, datatype, myname);
+
+    return error_code;
+}
+
+/* large count function */
+
+
+/*@
+    MPI_File_read_all_begin_c - Begin a split collective read using individual file pointer
+
+Input Parameters:
+. fh - file handle (handle)
+. count - number of elements in buffer (nonnegative integer)
+. datatype - datatype of each buffer element (handle)
+
+Output Parameters:
+. buf - initial address of buffer (choice)
+
+.N fortran
+@*/
+int MPI_File_read_all_begin_c(MPI_File fh, void *buf, MPI_Count count, MPI_Datatype datatype)
+{
+    assert(count <= INT_MAX);
+    int error_code;
+    static char myname[] = "MPI_FILE_READ_ALL_BEGIN";
+
+    error_code = MPIOI_File_read_all_begin(fh, (MPI_Offset) 0,
+                                           ADIO_INDIVIDUAL, buf, count, datatype, myname);
 
     return error_code;
 }
@@ -55,17 +93,15 @@ int MPI_File_read_all_begin(MPI_File fh, void *buf, int count,
 /* prevent multiple definitions of this routine */
 #ifdef MPIO_BUILD_PROFILING
 int MPIOI_File_read_all_begin(MPI_File fh,
-			      MPI_Offset offset,
-			      int file_ptr_type,
-			      void *buf,
-			      int count,
-			      MPI_Datatype datatype,
-			      char *myname)
+                              MPI_Offset offset,
+                              int file_ptr_type,
+                              void *buf, int count, MPI_Datatype datatype, char *myname)
 {
+    assert(count <= INT_MAX);
     int error_code;
     MPI_Count datatype_size;
     ADIO_File adio_fh;
-    void *xbuf=NULL, *e32_buf=NULL;
+    void *xbuf = NULL, *e32_buf = NULL;
 
     ROMIO_THREAD_CS_ENTER();
 
@@ -76,16 +112,14 @@ int MPIOI_File_read_all_begin(MPI_File fh,
     MPIO_CHECK_COUNT(adio_fh, count, myname, error_code);
     MPIO_CHECK_DATATYPE(adio_fh, datatype, myname, error_code);
 
-    if (file_ptr_type == ADIO_EXPLICIT_OFFSET && offset < 0)
-    {
-	error_code = MPIO_Err_create_code(MPI_SUCCESS, MPIR_ERR_RECOVERABLE,
-					  myname, __LINE__, MPI_ERR_ARG,
-					  "**iobadoffset", 0);
-	error_code = MPIO_Err_return_file(adio_fh, error_code);
-	goto fn_exit;
+    if (file_ptr_type == ADIO_EXPLICIT_OFFSET && offset < 0) {
+        error_code = MPIO_Err_create_code(MPI_SUCCESS, MPIR_ERR_RECOVERABLE,
+                                          myname, __LINE__, MPI_ERR_ARG, "**iobadoffset", 0);
+        error_code = MPIO_Err_return_file(adio_fh, error_code);
+        goto fn_exit;
     }
     /* --END ERROR HANDLING-- */
-    
+
     MPI_Type_size_x(datatype, &datatype_size);
 
     /* --BEGIN ERROR HANDLING-- */
@@ -94,11 +128,10 @@ int MPIOI_File_read_all_begin(MPI_File fh,
     MPIO_CHECK_NOT_SEQUENTIAL_MODE(adio_fh, myname, error_code);
 
     if (adio_fh->split_coll_count) {
-	error_code = MPIO_Err_create_code(MPI_SUCCESS, MPIR_ERR_RECOVERABLE,
-					  myname, __LINE__, MPI_ERR_IO, 
-					  "**iosplitcoll", 0);
-	error_code = MPIO_Err_return_file(adio_fh, error_code);
-	goto fn_exit;
+        error_code = MPIO_Err_create_code(MPI_SUCCESS, MPIR_ERR_RECOVERABLE,
+                                          myname, __LINE__, MPI_ERR_IO, "**iosplitcoll", 0);
+        error_code = MPIO_Err_return_file(adio_fh, error_code);
+        goto fn_exit;
     }
     MPIO_CHECK_COUNT_SIZE(adio_fh, count, datatype_size, myname, error_code);
     /* --END ERROR HANDLING-- */
@@ -106,32 +139,30 @@ int MPIOI_File_read_all_begin(MPI_File fh,
     adio_fh->split_coll_count = 1;
 
     xbuf = buf;
-    if (adio_fh->is_external32)
-    {
+    if (adio_fh->is_external32) {
         MPI_Aint e32_size = 0;
         error_code = MPIU_datatype_full_size(datatype, &e32_size);
         if (error_code != MPI_SUCCESS)
             goto fn_exit;
 
-        e32_buf = ADIOI_Malloc(e32_size*count);
-	xbuf = e32_buf;
+        e32_buf = ADIOI_Malloc(e32_size * count);
+        xbuf = e32_buf;
     }
 
     ADIO_ReadStridedColl(adio_fh, xbuf, count, datatype, file_ptr_type,
-			 offset, &adio_fh->split_status, &error_code);
+                         offset, &adio_fh->split_status, &error_code);
 
     /* --BEGIN ERROR HANDLING-- */
     if (error_code != MPI_SUCCESS)
-	error_code = MPIO_Err_return_file(adio_fh, error_code);
+        error_code = MPIO_Err_return_file(adio_fh, error_code);
     /* --END ERROR HANDLING-- */
 
     if (e32_buf != NULL) {
-        error_code = MPIU_read_external32_conversion_fn(xbuf, datatype,
-                count, e32_buf);
-	ADIOI_Free(e32_buf);
+        error_code = MPIU_read_external32_conversion_fn(buf, datatype, count, e32_buf);
+        ADIOI_Free(e32_buf);
     }
 
-fn_exit:
+  fn_exit:
     ROMIO_THREAD_CS_EXIT();
 
     return error_code;
