@@ -1,31 +1,23 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *  (C) 2001 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
 
 #include "mpidi_ch3_impl.h"
 #include "mpidrma.h"
 
 
-#undef FUNCNAME
-#define FUNCNAME MPIDI_CH3_Win_shared_query
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
-int MPIDI_CH3_SHM_Win_shared_query(MPID_Win * win_ptr, int target_rank, MPI_Aint * size,
+int MPIDI_CH3_SHM_Win_shared_query(MPIR_Win * win_ptr, int target_rank, MPI_Aint * size,
                                    int *disp_unit, void *baseptr)
 {
     int comm_size;
     int mpi_errno = MPI_SUCCESS;
-    MPIDI_STATE_DECL(MPID_STATE_MPIDI_CH3_WIN_SHARED_QUERY);
 
-    MPIDI_RMA_FUNC_ENTER(MPID_STATE_MPIDI_CH3_WIN_SHARED_QUERY);
+    MPIR_FUNC_ENTER;
 
     if (win_ptr->comm_ptr->node_comm == NULL) {
         mpi_errno = MPIDI_CH3U_Win_shared_query(win_ptr, target_rank, size, disp_unit, baseptr);
-        if (mpi_errno != MPI_SUCCESS) {
-            MPIR_ERR_POP(mpi_errno);
-        }
+        MPIR_ERR_CHECK(mpi_errno);
         goto fn_exit;
     }
 
@@ -44,7 +36,7 @@ int MPIDI_CH3_SHM_Win_shared_query(MPID_Win * win_ptr, int target_rank, MPI_Aint
         for (i = 0; i < comm_size; i++) {
             if (win_ptr->basic_info_table[i].size > 0) {
                 int local_i = win_ptr->comm_ptr->intranode_table[i];
-                MPIU_Assert(local_i >= 0 && local_i < win_ptr->comm_ptr->node_comm->local_size);
+                MPIR_Assert(local_i >= 0 && local_i < win_ptr->comm_ptr->node_comm->local_size);
                 *size = win_ptr->basic_info_table[i].size;
                 *disp_unit = win_ptr->basic_info_table[i].disp_unit;
                 *((void **) baseptr) = win_ptr->shm_base_addrs[local_i];
@@ -55,7 +47,7 @@ int MPIDI_CH3_SHM_Win_shared_query(MPID_Win * win_ptr, int target_rank, MPI_Aint
     }
     else {
         int local_target_rank = win_ptr->comm_ptr->intranode_table[target_rank];
-        MPIU_Assert(local_target_rank >= 0 &&
+        MPIR_Assert(local_target_rank >= 0 &&
                     local_target_rank < win_ptr->comm_ptr->node_comm->local_size);
         *size = win_ptr->basic_info_table[target_rank].size;
         *disp_unit = win_ptr->basic_info_table[target_rank].disp_unit;
@@ -63,7 +55,7 @@ int MPIDI_CH3_SHM_Win_shared_query(MPID_Win * win_ptr, int target_rank, MPI_Aint
     }
 
   fn_exit:
-    MPIDI_RMA_FUNC_EXIT(MPID_STATE_MPIDI_CH3_WIN_SHARED_QUERY);
+    MPIR_FUNC_EXIT;
     return mpi_errno;
 
   fn_fail:
@@ -71,16 +63,11 @@ int MPIDI_CH3_SHM_Win_shared_query(MPID_Win * win_ptr, int target_rank, MPI_Aint
 }
 
 
-#undef FUNCNAME
-#define FUNCNAME MPIDI_CH3_SHM_Win_free
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
-int MPIDI_CH3_SHM_Win_free(MPID_Win ** win_ptr)
+int MPIDI_CH3_SHM_Win_free(MPIR_Win ** win_ptr)
 {
     int mpi_errno = MPI_SUCCESS;
-    MPIDI_STATE_DECL(MPID_STATE_MPIDI_CH3_SHM_WIN_FREE);
 
-    MPIDI_RMA_FUNC_ENTER(MPID_STATE_MPIDI_CH3_SHM_WIN_FREE);
+    MPIR_FUNC_ENTER;
 
     if ((*win_ptr)->comm_ptr->node_comm == NULL) {
         goto fn_exit;
@@ -89,7 +76,7 @@ int MPIDI_CH3_SHM_Win_free(MPID_Win ** win_ptr)
     /* Free shared memory region */
     if ((*win_ptr)->shm_allocated) {
         /* free shm_base_addrs that's only used for shared memory windows */
-        MPIU_Free((*win_ptr)->shm_base_addrs);
+        MPL_free((*win_ptr)->shm_base_addrs);
 
         /* Only allocate and allocate_shared allocate new shared segments */
         if (((*win_ptr)->create_flavor == MPI_WIN_FLAVOR_SHARED ||
@@ -97,13 +84,12 @@ int MPIDI_CH3_SHM_Win_free(MPID_Win ** win_ptr)
             (*win_ptr)->shm_segment_len > 0) {
             /* detach from shared memory segment */
             mpi_errno =
-                MPIU_SHMW_Seg_detach((*win_ptr)->shm_segment_handle,
-                                     (char **) &(*win_ptr)->shm_base_addr,
+                MPL_shm_seg_detach((*win_ptr)->shm_segment_handle,
+                                     &(*win_ptr)->shm_base_addr,
                                      (*win_ptr)->shm_segment_len);
-            if (mpi_errno)
-                MPIR_ERR_POP(mpi_errno);
+            MPIR_ERR_CHECK(mpi_errno);
 
-            MPIU_SHMW_Hnd_finalize(&(*win_ptr)->shm_segment_handle);
+            MPL_shm_hnd_finalize(&(*win_ptr)->shm_segment_handle);
         }
     }
 
@@ -113,14 +99,14 @@ int MPIDI_CH3_SHM_Win_free(MPID_Win ** win_ptr)
     if (((*win_ptr)->create_flavor == MPI_WIN_FLAVOR_SHARED ||
          (*win_ptr)->create_flavor == MPI_WIN_FLAVOR_ALLOCATE) &&
         (*win_ptr)->shm_mutex && (*win_ptr)->shm_segment_len > 0) {
-        MPID_Comm *node_comm_ptr = NULL;
+        MPIR_Comm *node_comm_ptr = NULL;
 
         /* When allocating shared memory region segment, we need comm of processes
          * that are on the same node as this process (node_comm).
          * If node_comm == NULL, this process is the only one on this node, therefore
          * we use comm_self as node comm. */
         node_comm_ptr = (*win_ptr)->comm_ptr->node_comm;
-        MPIU_Assert(node_comm_ptr != NULL);
+        MPIR_Assert(node_comm_ptr != NULL);
 
         if (node_comm_ptr->rank == 0) {
             MPIDI_CH3I_SHM_MUTEX_DESTROY(*win_ptr);
@@ -128,23 +114,21 @@ int MPIDI_CH3_SHM_Win_free(MPID_Win ** win_ptr)
 
         /* detach from shared memory segment */
         mpi_errno =
-            MPIU_SHMW_Seg_detach((*win_ptr)->shm_mutex_segment_handle,
-                                 (char **) &(*win_ptr)->shm_mutex, sizeof(MPIDI_CH3I_SHM_MUTEX));
-        if (mpi_errno)
-            MPIR_ERR_POP(mpi_errno);
+            MPL_shm_seg_detach((*win_ptr)->shm_mutex_segment_handle,
+                                 (void **) &(*win_ptr)->shm_mutex, sizeof(MPIDI_CH3I_SHM_MUTEX));
+        MPIR_ERR_CHECK(mpi_errno);
 
-        MPIU_SHMW_Hnd_finalize(&(*win_ptr)->shm_mutex_segment_handle);
+        MPL_shm_hnd_finalize(&(*win_ptr)->shm_mutex_segment_handle);
     }
 
     /* Free shared memory region for window info */
     if ((*win_ptr)->info_shm_base_addr != NULL) {
-        mpi_errno = MPIU_SHMW_Seg_detach((*win_ptr)->info_shm_segment_handle,
-                                         (char **) &(*win_ptr)->info_shm_base_addr,
+        mpi_errno = MPL_shm_seg_detach((*win_ptr)->info_shm_segment_handle,
+                                         &(*win_ptr)->info_shm_base_addr,
                                          (*win_ptr)->info_shm_segment_len);
-        if (mpi_errno != MPI_SUCCESS)
-            MPIR_ERR_POP(mpi_errno);
+        MPIR_ERR_CHECK(mpi_errno);
 
-        MPIU_SHMW_Hnd_finalize(&(*win_ptr)->info_shm_segment_handle);
+        MPL_shm_hnd_finalize(&(*win_ptr)->info_shm_segment_handle);
 
         (*win_ptr)->basic_info_table = NULL;
     }
@@ -156,7 +140,7 @@ int MPIDI_CH3_SHM_Win_free(MPID_Win ** win_ptr)
     }
 
   fn_exit:
-    MPIDI_RMA_FUNC_EXIT(MPID_STATE_MPIDI_CH3_SHM_WIN_FREE);
+    MPIR_FUNC_EXIT;
     return mpi_errno;
 
   fn_fail:
