@@ -1,7 +1,6 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *  (C) 2008 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
 
 #ifndef HYDRA_SERVER_H_INCLUDED
@@ -13,14 +12,13 @@
 struct HYD_cmd {
     enum {
         HYD_CLEANUP,
-        HYD_CKPOINT,
         HYD_SIGNAL
     } type;
 
     int signum;
 };
 
-struct HYD_server_info {
+struct HYD_server_info_s {
     struct HYD_user_global user_global;
 
     char *base_path;
@@ -28,6 +26,7 @@ struct HYD_server_info {
     char *iface_ip_env_name;
     char *nameserver;
     char *localhost;
+    time_t time_start;
 
      HYD_status(*stdout_cb) (int pgid, int proxy_id, int rank, void *buf, int buflen);
      HYD_status(*stderr_cb) (int pgid, int proxy_id, int rank, void *buf, int buflen);
@@ -38,6 +37,9 @@ struct HYD_server_info {
     /* Process groups */
     struct HYD_pg pg_list;
 
+    /* Hash for fast proxy lookup */
+    struct HYD_proxy *proxy_hash;
+
     /* Cleanup */
     int cmd_pipe[2];
 
@@ -47,6 +49,6 @@ struct HYD_server_info {
 #endif                          /* ENABLE_PROFILING */
 };
 
-extern struct HYD_server_info HYD_server_info;
+extern struct HYD_server_info_s HYD_server_info;
 
 #endif /* HYDRA_SERVER_H_INCLUDED */
