@@ -1,9 +1,8 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *
- *  (C) 2003 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
+
 #include "mpi.h"
 #include "mpitest.h"
 #include <stdio.h>
@@ -49,8 +48,7 @@ int main(int argc, char *argv[])
             /* Create 3 more processes */
             MPI_Comm_spawn((char *) "./disconnect", MPI_ARGV_NULL, np,
                            MPI_INFO_NULL, 0, MPI_COMM_WORLD, &intercomm, MPI_ERRCODES_IGNORE);
-        }
-        else {
+        } else {
             intercomm = parentcomm;
         }
 
@@ -79,8 +77,7 @@ int main(int argc, char *argv[])
              * errs += err;
              * }
              */
-        }
-        else {
+        } else {
             IF_VERBOSE(("child rank %d alive.\n", rank));
             /* Child */
             if (size != np) {
@@ -92,7 +89,7 @@ int main(int argc, char *argv[])
             IF_VERBOSE(("disconnecting communicator\n"));
             MPI_Comm_disconnect(&intercomm);
 
-            /* Send the errs back to the master process */
+            /* Send the errs back to the parent process */
             /* Errors cannot be sent back to the parent because there is no
              * communicator connected to the parent */
             /*MPI_Ssend(&errs, 1, MPI_INT, 0, 1, intercomm); */
@@ -103,13 +100,13 @@ int main(int argc, char *argv[])
          * if both call MTest_Finalize */
         if (parentcomm == MPI_COMM_NULL) {
             MTest_Finalize(errs);
+        } else {
+            MPI_Finalize();
         }
-    }
-    else {
+    } else {
         MTest_Finalize(errs);
     }
 
     IF_VERBOSE(("calling finalize\n"));
-    MPI_Finalize();
-    return 0;
+    return MTestReturnValue(errs);
 }
